//! Module for color palettes you might wish to use with your indexed color
//! images.

/// A palette for NES colors.
///
/// There's no official RGB palette for the NES since the system actually used
/// "YPbPr" color. Instead, there's a bunch of different palettes you can used
/// that will look more or less like what you might remember from when you were
/// a kid. This palette here is based on [this
/// image](http://www.firebrandx.com/graphics/nespalettes/nespalette-pvm-style-d93.png).
pub static NES_PALETTE: [u32; 64] = [
  // row 0
  rgb32!(105, 107, 99), // 0
  rgb32!(0, 23, 116),   // 1
  rgb32!(30, 0, 135),   // 2
  rgb32!(52, 0, 115),   // 3
  rgb32!(86, 0, 87),    // 4
  rgb32!(94, 0, 19),    // 5
  rgb32!(83, 26, 0),    // 6
  rgb32!(59, 36, 0),    // 7
  rgb32!(36, 48, 0),    // 8
  rgb32!(6, 58, 0),     // 9
  rgb32!(0, 63, 0),     // A
  rgb32!(0, 59, 30),    // B
  rgb32!(0, 58, 78),    // C
  rgb32!(0, 0, 0),      // D
  rgb32!(0, 0, 0),      // E
  rgb32!(0, 0, 0),      // F
  // row 1
  rgb32!(185, 187, 179), // 0
  rgb32!(20, 83, 185),   // 1
  rgb32!(77, 44, 218),   // 2
  rgb32!(103, 30, 222),  // 3
  rgb32!(152, 24, 156),  // 4
  rgb32!(157, 35, 68),   // 5
  rgb32!(160, 62, 0),    // 6
  rgb32!(141, 85, 0),    // 7
  rgb32!(101, 109, 0),   // 8
  rgb32!(44, 121, 0),    // 9
  rgb32!(0, 129, 0),     // A
  rgb32!(0, 125, 66),    // B
  rgb32!(0, 120, 138),   // C
  rgb32!(0, 0, 0),       // D
  rgb32!(0, 0, 0),       // E
  rgb32!(0, 0, 0),       // F
  // row 2
  rgb32!(255, 255, 255), // 0
  rgb32!(105, 168, 255), // 1
  rgb32!(150, 145, 255), // 2
  rgb32!(178, 138, 250), // 3
  rgb32!(234, 125, 250), // 4
  rgb32!(243, 123, 199), // 5
  rgb32!(242, 142, 89),  // 6
  rgb32!(230, 173, 39),  // 7
  rgb32!(215, 200, 5),   // 8
  rgb32!(144, 223, 7),   // 9
  rgb32!(100, 229, 60),  // A
  rgb32!(69, 226, 125),  // B
  rgb32!(72, 213, 217),  // C
  rgb32!(78, 80, 72),    // D
  rgb32!(0, 0, 0),       // E
  rgb32!(0, 0, 0),       // F
  // row 3
  rgb32!(255, 255, 255), // 0
  rgb32!(210, 234, 255), // 1
  rgb32!(226, 226, 255), // 2
  rgb32!(233, 216, 255), // 3
  rgb32!(245, 210, 255), // 4
  rgb32!(248, 217, 234), // 5
  rgb32!(250, 222, 185), // 6
  rgb32!(249, 232, 155), // 7
  rgb32!(243, 242, 140), // 8
  rgb32!(211, 250, 145), // 9
  rgb32!(184, 252, 168), // A
  rgb32!(174, 250, 202), // B
  rgb32!(202, 243, 243), // C
  rgb32!(190, 192, 184), // D
  rgb32!(0, 0, 0),       // E
  rgb32!(0, 0, 0),       // F
];

/// Colors from the Sharecart1000
/// [spec](http://sharecart1000.com/img/SHARECART1000guide.png).
///
/// The colors are ordered by their appearance within the mini "color scale"
/// grid toward the bottom. I've given each color a name so that you can kinda
/// keep track of what's going on. As with most things sharecart, it's largely
/// all nonsense and you should just go with it.
pub static SHARECART_PALETTE: [u32; 15] = [
  /// hardwood
  rgb32!(50, 45, 35),
  /// dark brown
  rgb32!(95, 57, 31),
  /// light brown
  rgb32!(148, 74, 28),
  /// gray
  rgb32!(144, 144, 152),
  /// off white
  rgb32!(252, 254, 252),
  /// slate
  rgb32!(73, 84, 65),
  /// dark tan
  rgb32!(148, 146, 76),
  /// light tan
  rgb32!(220, 198, 124),
  /// pine green
  rgb32!(28, 70, 36),
  /// grass green
  rgb32!(60, 136, 68),
  /// smoke
  rgb32!(164, 218, 196),
  /// deep ocean
  rgb32!(28, 62, 68),
  /// ocean
  rgb32!(4, 92, 156),
  /// dark sky blue
  rgb32!(4, 174, 204),
  /// light sky blue
  rgb32!(108, 250, 220),
];

// ENHANCE: ANSI terminal color palettes?
// http://2.bp.blogspot.com/-pHKF2JFbs6s/UiL9cHwzuvI/AAAAAAAAACg/XPE6eJ5ajrE/s1600/ansi_palettes.png
